@@ -1,5 +1,6 @@
 import warnings
-from typing import Iterable, List, Literal, Union, overload
+from typing import (Any, Iterable, List, Literal, Union,
+                    overload)
 
 from numpy import uint32
 
@@ -12,6 +13,8 @@ from .eorzea_time import EorzeaTime
 from .errors import WeatherRateDataError
 
 Lang = Union[Literal['en', 'ja', 'de', 'fr'], EorzeaLang]
+ValidPlaceName = Union[str, EorzeaPlaceName]
+Timestamp = Union[EorzeaTime, float, int]
 
 
 class EorzeaWeather:
@@ -30,8 +33,8 @@ class EorzeaWeather:
     @classmethod
     def forecast(
         cls,
-        place_name: Union[str, EorzeaPlaceName],
-        timestamp: Iterable[Union[int, float, EorzeaTime]],
+        place_name: ValidPlaceName,
+        timestamp: Iterable[Timestamp],
         lang: Lang = EorzeaLang.EN,
         strict: bool = True,
         raw: Literal[False] = False
@@ -41,30 +44,20 @@ class EorzeaWeather:
     @classmethod
     def forecast(
         cls,
-        place_name: Union[str, EorzeaPlaceName],
-        timestamp: Iterable[Union[int, float, EorzeaTime]],
+        place_name: ValidPlaceName,
+        timestamp: Iterable[Timestamp],
         lang: Lang = EorzeaLang.EN,
         strict: bool = True,
-        raw: Literal[True] = True
+        *,
+        raw: Literal[True]
     ) -> List[int]: ...
 
     @overload
     @classmethod
     def forecast(
         cls,
-        place_name: Union[str, EorzeaPlaceName],
-        timestamp: Iterable[Union[int, float, EorzeaTime]],
-        lang: Lang = EorzeaLang.EN,
-        strict: bool = True,
-        raw: bool = False
-    ) -> Union[List[int], List[str]]: ...
-
-    @overload
-    @classmethod
-    def forecast(
-        cls,
-        place_name: Union[str, EorzeaPlaceName],
-        timestamp: Union[int, float, EorzeaTime],
+        place_name: ValidPlaceName,
+        timestamp: Timestamp,
         lang: Lang = EorzeaLang.EN,
         strict: bool = True,
         raw: Literal[False] = False
@@ -74,33 +67,23 @@ class EorzeaWeather:
     @classmethod
     def forecast(
         cls,
-        place_name: Union[str, EorzeaPlaceName],
-        timestamp: Union[int, float, EorzeaTime],
+        place_name: ValidPlaceName,
+        timestamp: Timestamp,
         lang: Lang = EorzeaLang.EN,
         strict: bool = True,
-        raw: Literal[True] = True
+        *,
+        raw: Literal[True]
     ) -> int: ...
 
-    @overload
     @classmethod
     def forecast(
         cls,
-        place_name: Union[str, EorzeaPlaceName],
-        timestamp: Union[int, float, EorzeaTime],
+        place_name: ValidPlaceName,
+        timestamp: Union[Timestamp, Iterable[Timestamp]],
         lang: Lang = EorzeaLang.EN,
         strict: bool = True,
         raw: bool = False
-    ) -> Union[str, int]: ...
-
-    @classmethod
-    def forecast(
-            cls,
-            place_name: Union[str, EorzeaPlaceName],
-            timestamp: Union[int, float, EorzeaTime, Iterable[Union[int, float, EorzeaTime]]],
-            lang: Lang = EorzeaLang.EN,
-            strict: bool = True,
-            raw: bool = False
-    ) -> Union[str, int, List[str], List[int]]:
+    ):
         """Forecast Eorzea weather by place
 
         Parameters
@@ -138,60 +121,59 @@ class EorzeaWeather:
         assert type(place_name) is EorzeaPlaceName
         weather_rate = _territory_weather[place_name.index]
 
-        if isinstance(timestamp, Iterable):
-            if _check_iterable_timestamp(timestamp):
-                targets = (
-                    _calculate_forecast_target(t)
-                    for t in timestamp
-                )
-                result = [
-                    _generate_result(target, weather_rate, lang, raw=raw)
-                    for target in targets
-                ]
-
-                return result  # type: ignore
-
-        if isinstance(timestamp, (float, int, EorzeaTime)):
+        def make_result(timestamp: EorzeaTime):
             target = _calculate_forecast_target(timestamp)
-            result = _generate_result(target, weather_rate, lang, raw=raw)
-
+            result = _generate_result(target, weather_rate)
+            if not raw:
+                result = cls.get_weather(result, lang)
+                assert result
             return result
 
-        raise TypeError(
-            "timestamp should be type of Iterable[Union[int, flaot]], int, float."
-        )
+        if isinstance(timestamp, Iterable):
+            results = []
+
+            for t in timestamp:
+                t = _ensure_timestamp(t)
+                result = make_result(t)
+                results.append(result)
+
+            return results
+
+        timestamp = _ensure_timestamp(timestamp)
+        result = make_result(timestamp)
+        return result
 
     @staticmethod
     def get_weather(index: int, lang: Lang):
         return _weather[index][lang]
 
 
-def _check_iterable_timestamp(timestamp: Iterable[Union[int, float, EorzeaTime]]) -> bool:
-    return all(
-        type(t) is int or type(t) is float or type(t) is EorzeaTime
-        for t in timestamp
+def _ensure_timestamp(timestamp: Any) -> EorzeaTime:
+    if type(timestamp) is EorzeaTime:
+        return timestamp
+    if type(timestamp) in (float, int):
+        warnings.warn(
+            "timestamp in float and int type is deprecated. It will be unsupported at 2.5.0. Please use EorzeaTime instead.",
+            DeprecationWarning
+        )
+        return EorzeaTime(timestamp=timestamp)
+    raise TypeError(
+        "timestamp should be type of int, float, EorzeaTime."
     )
 
 
-@overload
-def _generate_result(target: int, weather_rate: int, lang: str, raw: Literal[True]) -> int: ...
-@overload
-def _generate_result(target: int, weather_rate: int, lang: str, raw: Literal[False]) -> str: ...
-@overload
-def _generate_result(target: int, weather_rate: int, lang: str, raw: bool = False) -> Union[int, str]: ...
-
-
-def _generate_result(target: int, weather_rate: int, lang: str, raw: bool = False) -> Union[int, str]:
+def _generate_result(
+    target: int,
+    weather_rate: int
+) -> int:
     for rate, weather in _weather_rate[weather_rate]:
         if target < rate:
-            return weather if raw else _weather[weather][lang]
+            return weather
 
-    raise WeatherRateDataError(
-        "No matched rate in data. Please contact with developer."
-    )
+    raise WeatherRateDataError("No matched rate in data. Please contact with developer.")
 
 
-def _calculate_forecast_target(local_timestamp: Union[int, float, EorzeaTime]) -> int:
+def _calculate_forecast_target(the_time: EorzeaTime) -> int:
     """
     Thanks to Rogueadyn's SaintCoinach library for this calculation
     --------------
@@ -203,8 +185,8 @@ def _calculate_forecast_target(local_timestamp: Union[int, float, EorzeaTime]) -
 
     Parameters
     ----------
-    local_timestamp : float
-        local timestamp
+    the_time : EorzeaTime
+        time to calculate the target.
 
     Returns
     -------
@@ -212,14 +194,11 @@ def _calculate_forecast_target(local_timestamp: Union[int, float, EorzeaTime]) -
         weather period start
     """
 
-    if isinstance(local_timestamp, EorzeaTime):
-        local_timestamp = local_timestamp.get_unix_time()
-    else:
-        warnings.warn("timestamp in float and int type would be deprecated at 2.5.0", DeprecationWarning)
+    ts = the_time.get_unix_time()
 
-    bell = local_timestamp / 175
+    bell = ts / 175
     increment = uint32(bell + 8 - (bell % 8)) % 24
-    total_days = uint32(local_timestamp / 4200)
+    total_days = uint32(ts / 4200)
     calc_base = total_days * 0x64 + increment
     step1 = uint32(calc_base << 0xB) ^ calc_base
     step2 = uint32(step1 >> 8) ^ step1
